# ----------------------------------------------------------
# AB: oct 2005, revue oct 2006
# Fonction :
# Crer des groupes de valeurs parmi les rsultats
# lus sur un fichier de rsultats
# Pour chaque poly, on fait la somme de ce qu'il recoit au m2
# de tous les autres et on fait des groupes de valeurs
# Arguments:
# - fileres: nom  (ou pathname) du fichier de rsultats
# - logr: TRUE si on veut crer des groupes  partir des logs
#  - n.gp: nbre max de groupes voulus
# - meth: "pam" si on veut des groupes crs par 'pam',
# (les bornes de ces groupes seront donc variables
# selon le poly metteur,
# cf cette fonction R dans le package 'cluster').
# Par dfaut: n.gp groupes d'gale tendue,
# cette tendue tant retourne dans le composant 'scale'.
# Contrainte: n.gp = min((npoly-1), n.gp)
# igr, vecteur des indices sur le fichier de resultats
# des valeurs dont on veut les groupes
# datafile: si donn, trac des polys avec une couleur differente
# selon leur groupe
# Dans ce cas, il faut aussi donner 'format'
# et delim et
# plotfile, la racine des postscripts crs
# (ils seront sur X11 si plotfile =null)
# ... arguments passs tels quels a plot
# Retour: une liste qui comprend:
# call: le match.call
# Pour chaque valeur de igr, un composant dont le nom est
# la valeur correspondante de igr,
# qui contient :
# res: matrice dont les labels de ligne sont les identif de ploys
# et dont les colonnes sont la somme des valeurs que le poly
# emet (recoit) et le groupe auquel il est affect
# En plus, 
# - si la mthode n'est pas pam:   scale: l'etendue des groupe
# - si la mthode est pam: tout ce que renvoie cette fonction
# ---------------------------------------------------
gpe = function( fileres, logr=FALSE, n.gp=3,
   meth="",igr=c(3,4),
  datafile=NULL, format=1, delim=" ", plotfile="gp",
   ...)
{

  library("cluster") # pour 'pam'

# ----------------------------------------------------------
# Fonction :
# Faire les groupes de valeurs 
# ------------------------------------------------------------  
  fairegpe = function( meth, rmatrice , n.gp) {
    retour=list()
    
    if (meth == "pam") {
      retour = pam(rmatrice , k=n.gp, diss=FALSE)
      retour$res = matrix(retour$clustering, ncol=1)
      dimnames(retour$res)=list(dimnames(rmatrice)[[1]], "gp")
    }
    else {
# groupes d'gale tendue
      et = range(rmatrice[!is.na(rmatrice)])
      largeur = (et[2]-et[1]) / n.gp
      gpmatrice <- rmatrice
      binf = et[1]

      for (legp in 1:n.gp) {
        bsup = binf + largeur
        gpmatrice[ (rmatrice >= binf) & (rmatrice < bsup) & !is.na(rmatrice)] = legp
        binf = bsup
      }
      gpmatrice[ (rmatrice==et[2]) ] = n.gp
      retmat = matrix(c( rmatrice, gpmatrice), ncol= (ncol(rmatrice)+1))
      dimnames(retmat ) = list(dimnames(rmatrice)[[1]], c("sum", "gp"))
      retour = list(scale=largeur, res=retmat)
    } # fin groupes d'gale tendue
    return(retour)
  } # fin faire gpe
# --------------------------------------------------------
  # Fonction utilisee via un apply:
  # sommer les icoliemes valeurs de res relatives  un
  # meme poly emetteur
# --------------------------------------------------------
  sommer = function(x, res, icol)    {
      r = sum(res[res[,2]==x[1], icol])
      return(c(x[1], r))
} # fin sommer


# -----------------------------------------------------
# Trace d'une matrice de groupes
# -----------------------------------------------------
  unplgpe<- function( polygpe,  lespoly,  milieux, ask=TRUE, ...) {
# Extraction des polygones dont le no est dans polyygpe:
#    polyygpe: matrice dont la col nomme "gp" est le groupe
# et dont les labels de lignes sont les nos de poly    
    polye<- list()
    ipoly <- dimnames(polygpe)[[1]]
    npoly <- nrow(polygpe)
    for (i in 1:npoly) 
      polye[[i]] <-lespoly[[ipoly[i]]]

# Rajout de la classe:
    class(polye) <- "listpoly"

# Pour crer le cadre:
    l <- range.listpoly(polye)
    etendue <- max(apply(l,2,diff))

# Crer les couleurs:
    r <- rainbow(max(polygpe[,"gp"]))

    if (ask)
      par(ask=TRUE)
    else
      par(ask=FALSE)
    par(pty="s") # Pour avoir la mme chelle sur les 2 axes

# Pour crire la correspondance des groupes et des couleurs:
    # Avec pam, les groupes ne sont pas dans l'ordre
    # de la diffusion
    if (!is.null(milieux)) {
      zz= order(milieux[,1])
      rz= r[zz]
    }
    else
      rz=r

    legendcol=c()
    legendtext=c()
    for (i in 1:length(rz)) {
      legendcol=c(legendcol, rz[i])
      legendtext=c(legendtext, as.character(i))
    }

      
# Trac du cadre: (pty='s' => un carr)
    # Rajouter un peu pour la lgende
    largeur=etendue/20
      plot( x=c(l[1,"xrange"], l[1,"xrange"]+etendue+largeur),
           y=c(l[1,"yrange"], l[1,"yrange"]+etendue+largeur),
           type="n", xlab="", ylab="")
    legend(x=l[2,"xrange"], y=l[2,"yrange"],
           legendtext, text.col=legendcol)
      for (j in (1:npoly)) {
        attr( polye[[j]], "couleur") <- r[polygpe[j, "gp"]]
        plot.poly(polye[[j]], add=TRUE, ...)
# Rajout du no de poly au milieu des parcelles:
        text(x=mean(polye[[j]][,1]),
             y=mean(polye[[j]][,2]),
             labels=dimnames(polygpe)[[1]][j])
      } # fin j

    invisible()
  } # fin unplgpe

                                        # --------------------------------------------------------
# Fonction principale
# --------------------------------------------------------
# --------------------------------------------
    # lire les fichiers
# --------------------------------------------
#-------------------------
# Lire les polygones si trace des groupes:
#-------------------------
      if (!is.null(datafile)) {
        if (format==0)
          stop("No format given\n")

if (format==1)
donnees = readpoly1(filename = datafile, delim=delim)
else
  donnees = readpoly2(filename = datafile, delim=delim)
    
      } # fin datafile


# --------------------------------------------
  # Lire le fichier des rsultats
# --------------------------------------------
  res = read.table(fileres, header=FALSE,skip=1)
# Prendre eventuellement les log
  if (logr == TRUE)
    res[,igr] = log(res[,igr])
  
  dimnames(res)[[2]][1:2]=    c("p1", "p2")

  z = unique (res["p2"])
  z=as.vector(z[,1])
#z: les nos de poly emetteurs

  # Prparer le retour
# --------------------------------------------
  retour = list(call=match.call())
  
  #-------------------------
  for (i in igr) {
  # Faire une matrice  2 colonnes:
  # la 1iere est le no de poly et la 2ieme est la somme
  # des valeurs de res o ce poly est en colonne "p1"
  somres = matrix(c(z, rep(NaN, length(z))), ncol=2)
  a = t(apply(somres, 1, sommer, res, i))
  somres = matrix(a[,2], ncol=1)
  dimnames(somres) = list(z, NULL)

  
# Faire les groupes de valeurs :
  polygpe= fairegpe ( meth, somres , n.gp)
  retour[[as.character(i)]] =polygpe
  
  # Les tracs
  if (!is.null(datafile))
    {
## Ouvrir X11 en rajoutant des couleurs
#  si aucun device encore ouvert
# AB: 21/11/2005 Pas besoin: c'est automatique et comme colortype
  # ne marche pas avec R.2., je l'te
#  if (dev.cur() == 1) X11(colortype="pseudo.cube" )
      if (!is.null(plotfile)) {
        nomplot = paste(plotfile,as.character(i),".ps",sep="")
        postscript(nomplot)
        ask=FALSE
      }
      
    unplgpe(polygpe$res, donnees, polygpe$medoids ,ask=ask, ...)
      if (!is.null(plotfile)) {
        cat("Create the file ", nomplot, "\n")
        dev.off()
      }
      
} # fin (!is.null(datafile))
  
  
}
  
# Enchainer sur plgpe, pour le trace, les polys
# ayant t lus par litpoly.R

  return(retour)
}
